/**
  ******************************************************************************
    @file    tftlcd_mcufriend_lib.cpp
    @author  JSC electronics
    @version V1.0
    @date    28.08.2017
    @brief   Test  displeje 1487762760
             Pro test byla použita knihovna z https://github.com/prenticedavid/MCUFRIEND_kbv
*/

#include "tftlcd_mcufriend_lib.h"

// promenne - souradnice
int buf[318];
int x, x2;
int y, y2;
int r;

UTFTGLUE myGLCD(0x0154, A2, A1, A3, A4, A0); //nastaveni pinu pro knihovnu UTFTGLUE

/**
    @brief  draw_frame
           vykresli ramecek a popisky
    @param  none
    @retval none
*/

void draw_frame(void) {
  myGLCD.clrScr();

  myGLCD.setColor(255, 0, 0);
  myGLCD.fillRect(0, 0, 319, 13);
  myGLCD.setColor(64, 64, 64);
  myGLCD.fillRect(0, 226, 319, 239);
  myGLCD.setColor(255, 255, 255);
  myGLCD.setBackColor(255, 0, 0);
  myGLCD.print("* Test TFTLCD - MCUFRIEND probiha *", CENTER, 1);
  myGLCD.setBackColor(64, 64, 64);
  myGLCD.setColor(255, 255, 0);
  myGLCD.print("upraveny priklad z knihovny MCUFRIEND_kbv", CENTER, 227);

  myGLCD.setColor(0, 0, 255);
  myGLCD.drawRect(0, 14, 319, 225);
}


/**
    @brief  crosshairs
            vykresli mrizku
    @param  none
    @retval none
*/

void crosshairs(void) {
  myGLCD.setColor(0, 0, 255);
  myGLCD.setBackColor(0, 0, 0);
  myGLCD.drawLine(159, 15, 159, 224);
  myGLCD.drawLine(1, 119, 318, 119);
  for (int i = 9; i < 310; i += 10)
    myGLCD.drawLine(i, 117, i, 121);
  for (int i = 19; i < 220; i += 10)
    myGLCD.drawLine(157, i, 161, i);

}


/**
    @brief  sin_cos_tan
            vykresli tyto tri funkce
    @param  none
    @retval none
*/

void sin_cos_tan(void) {
  myGLCD.setColor(0, 255, 255);
  myGLCD.print("Sin", 5, 15);
  for (int i = 1; i < 318; i++)
  {
    myGLCD.drawPixel(i, 119 + (sin(((i * 1.13) * 3.14) / 180) * 95));
  }

  myGLCD.setColor(255, 0, 0);
  myGLCD.print("Cos", 5, 27);
  for (int i = 1; i < 318; i++)
  {
    myGLCD.drawPixel(i, 119 + (cos(((i * 1.13) * 3.14) / 180) * 95));
  }

  myGLCD.setColor(255, 255, 0);
  myGLCD.print("Tan", 5, 39);
  for (int i = 1; i < 318; i++)
  {
    myGLCD.drawPixel(i, 119 + (tan(((i * 1.13) * 3.14) / 180)));
  }

  delay(2000);

  myGLCD.setColor(0, 0, 0);
  myGLCD.fillRect(1, 15, 318, 224);
  myGLCD.setColor(0, 0, 255);
  myGLCD.setBackColor(0, 0, 0);
  myGLCD.drawLine(159, 15, 159, 224);
  myGLCD.drawLine(1, 119, 318, 119);
}


/**
    @brief  sinusoid
            vykresli pohyblivou sinusoidu
    @param  none
    @retval none
*/

void sinusoid(void) {
  x = 1;
  for (int i = 1; i < (318 * 20); i++)
  {
    x++;
    if (x == 319)
      x = 1;
    if (i > 319)
    {
      if ((x == 159) || (buf[x - 1] == 119))
        myGLCD.setColor(0, 0, 255);
      else
        myGLCD.setColor(0, 0, 0);
      myGLCD.drawPixel(x, buf[x - 1]);
    }
    myGLCD.setColor(0, 255, 255);
    y = 119 + (sin(((i * 1.1) * 3.14) / 180) * (90 - (i / 100)));
    myGLCD.drawPixel(x, y);
    buf[x - 1] = y;
  }

  delay(2000);

  myGLCD.setColor(0, 0, 0);
  myGLCD.fillRect(1, 15, 318, 224);
}


/**
    @brief  filled_rectangles
            vykresli barevne obdelniky
    @param  none
    @retval none
*/

void filled_rectangles(void) {
  for (int i = 1; i < 6; i++)
  {
    switch (i)
    {
      case 1:
        myGLCD.setColor(255, 0, 255);
        break;
      case 2:
        myGLCD.setColor(255, 0, 0);
        break;
      case 3:
        myGLCD.setColor(0, 255, 0);
        break;
      case 4:
        myGLCD.setColor(0, 0, 255);
        break;
      case 5:
        myGLCD.setColor(255, 255, 0);
        break;
    }
    myGLCD.fillRect(70 + (i * 20), 30 + (i * 20), 130 + (i * 20), 90 + (i * 20));
  }

  delay(2000);

  myGLCD.setColor(0, 0, 0);
  myGLCD.fillRect(1, 15, 318, 224);
}


/**
    @brief  rounded_rectangles
            vykresli zaoblene obdelniky
    @param  none
    @retval none
*/

void rounded_rectangles(void) {
  for (int i = 1; i < 6; i++)
  {
    switch (i)
    {
      case 1:
        myGLCD.setColor(255, 0, 255);
        break;
      case 2:
        myGLCD.setColor(255, 0, 0);
        break;
      case 3:
        myGLCD.setColor(0, 255, 0);
        break;
      case 4:
        myGLCD.setColor(0, 0, 255);
        break;
      case 5:
        myGLCD.setColor(255, 255, 0);
        break;
    }
    myGLCD.fillRoundRect(190 - (i * 20), 30 + (i * 20), 250 - (i * 20), 90 + (i * 20));
  }

  delay(2000);

  myGLCD.setColor(0, 0, 0);
  myGLCD.fillRect(1, 15, 318, 224);

}

/**
    @brief  filled_circles
            vykresli barevne kruhy
    @param  none
    @retval none
*/

void filled_circles(void) {
  for (int i = 1; i < 6; i++)
  {
    switch (i)
    {
      case 1:
        myGLCD.setColor(255, 0, 255);
        break;
      case 2:
        myGLCD.setColor(255, 0, 0);
        break;
      case 3:
        myGLCD.setColor(0, 255, 0);
        break;
      case 4:
        myGLCD.setColor(0, 0, 255);
        break;
      case 5:
        myGLCD.setColor(255, 255, 0);
        break;
    }
    myGLCD.fillCircle(100 + (i * 20), 60 + (i * 20), 30);
  }

  delay(2000);

  myGLCD.setColor(0, 0, 0);
  myGLCD.fillRect(1, 15, 318, 224);
}

/**
    @brief  lines
            vykresli cary
    @param  none
    @retval none
*/

void lines(void) {
  myGLCD.setColor (255, 0, 0);
  for (int i = 15; i < 224; i += 5)
  {
    myGLCD.drawLine(1, i, (i * 1.44) - 10, 224);
  }
  myGLCD.setColor (255, 0, 0);
  for (int i = 224; i > 15; i -= 5)
  {
    myGLCD.drawLine(318, i, (i * 1.44) - 11, 15);
  }
  myGLCD.setColor (0, 255, 255);
  for (int i = 224; i > 15; i -= 5)
  {
    myGLCD.drawLine(1, i, 331 - (i * 1.44), 15);
  }
  myGLCD.setColor (0, 255, 255);
  for (int i = 15; i < 224; i += 5)
  {
    myGLCD.drawLine(318, i, 330 - (i * 1.44), 224);
  }

  delay(2000);

  myGLCD.setColor(0, 0, 0);
  myGLCD.fillRect(1, 15, 318, 225);
}

/**
    @brief  random_circles
            vykresli nahodne kruhy
    @param  none
    @retval none
*/

void random_circles(void) {
  for (int i = 0; i < 100; i++)
  {
    myGLCD.setColor(random(255), random(255), random(255));
    x = 32 + random(256);
    y = 45 + random(146);
    r = random(30);
    myGLCD.drawCircle(x, y, r);
  }

  delay(2000);

  myGLCD.setColor(0, 0, 0);
  myGLCD.fillRect(1, 15, 318, 224);
}

/**
    @brief  random_rectangles
            vykresli nahodne obdelniky
    @param  none
    @retval none
*/

void random_rectangles(void) {
  for (int i = 0; i < 100; i++)
  {
    myGLCD.setColor(random(255), random(255), random(255));
    x = 2 + random(316);
    y = 16 + random(207);
    x2 = 2 + random(316);
    y2 = 16 + random(207);
    myGLCD.drawRect(x, y, x2, y2);
  }

  delay(2000);

  myGLCD.setColor(0, 0, 0);
  myGLCD.fillRect(1, 15, 318, 224);
}

/**
    @brief  random_rounded_rectangles
            vykresli nahodne zaoblene obdelniky
    @param  none
    @retval none
*/

void random_rounded_rectangles(void) {
  for (int i = 0; i < 100; i++)
  {
    myGLCD.setColor(random(255), random(255), random(255));
    x = 2 + random(316);
    y = 16 + random(207);
    x2 = 2 + random(316);
    y2 = 16 + random(207);
    myGLCD.drawRoundRect(x, y, x2, y2);
  }

  delay(2000);

  myGLCD.setColor(0, 0, 0);
  myGLCD.fillRect(1, 15, 318, 224);

  for (int i = 0; i < 100; i++)
  {
    myGLCD.setColor(random(255), random(255), random(255));
    x = 2 + random(316);
    y = 16 + random(209);
    x2 = 2 + random(316);
    y2 = 16 + random(209);
    myGLCD.drawLine(x, y, x2, y2);
  }

  delay(2000);

  myGLCD.setColor(0, 0, 0);
  myGLCD.fillRect(1, 15, 318, 224);

  for (int i = 0; i < 10000; i++)
  {
    myGLCD.setColor(random(255), random(255), random(255));
    myGLCD.drawPixel(2 + random(316), 16 + random(209));
  }

  delay(2000);
}

/**
    @brief  test_results
            ukonceni a vyhodnoceni testu
    @param  none
    @retval none
*/

void test_results(void) {
  myGLCD.fillScr(0, 0, 255);
  myGLCD.setColor(255, 0, 0);
  myGLCD.fillRoundRect(80, 70, 239, 169);

  myGLCD.setColor(255, 255, 255);
  myGLCD.setBackColor(255, 0, 0);
  myGLCD.print("Test dokoncen!", CENTER, 93);
  myGLCD.print("Restart probehne", CENTER, 119);
  myGLCD.print("za par sekund...", CENTER, 132);

  myGLCD.setColor(0, 255, 0);
  myGLCD.setBackColor(0, 0, 255);
  myGLCD.print("Delka testu: (s)", CENTER, 210);
  myGLCD.printNumI(millis() / 1000, CENTER, 225);

  delay (10000);
}

/**
    @brief  init
            nastaveni vstupů, výstupů a displeje
    @param  none
    @retval none
*/

void init_lcd(void) {
  randomSeed(analogRead(5));
  pinMode(A0, OUTPUT);
  digitalWrite(A0, HIGH);

  // nastaveni displeje
  myGLCD.InitLCD();
  myGLCD.setFont(SmallFont);
}


/**
    @brief  test_function
            spousti funkce testu
    @param  none
    @retval none
*/

void test_function() {

  draw_frame();
  crosshairs();
  sin_cos_tan();
  sinusoid();
  filled_rectangles();
  rounded_rectangles();
  filled_circles();
  lines();
  random_circles();
  random_rectangles();
  random_rounded_rectangles();

  test_results();
}



